/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.area.plot;

import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import java.beans.IntrospectionException;
import java.util.Date;
import nl.cloudfarming.client.geoviewer.LayerObject;
import nl.cloudfarming.client.model.Plot;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.openide.ErrorManager;
import org.openide.nodes.Node.Property;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.Exceptions;

/**
 *
 * @author Timon Veenstra
 */
public class PlotLayerObject implements LayerObject<MultiPolygon, Plot> {

    private final MultiPolygon geometry;
    private final Plot plot;

    public PlotLayerObject(Plot plot) throws IntrospectionException {
//        setDisplayName(plot.getName());
        try {
            GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory(null);
            WKTReader reader = new WKTReader(geometryFactory);
            geometry = (MultiPolygon) reader.read(plot.getWKT());
            this.plot = plot;
        } catch (ParseException ex) {
            Exceptions.printStackTrace(ex);
            throw new RuntimeException(ex);
        }
    }

    @Override
    public MultiPolygon getGeometry() {
        return this.geometry;
    }

    @Override
    public String getKey() {
        return Long.toString(this.plot.getId());
    }

    @Override
    public String getTooltipText() {
        return Long.toString(this.plot.getId()) + " " + this.plot.getName();
    }

    @Override
    public Long getKeyAttributeValue() {
        return this.plot.getId();
    }

    @Override
    public Long getId() {
        return this.plot.getId();
    }

    @Override
    public Plot getObject() {
        return this.plot;
    }

//
//    public Sheet createSheet() {
//
//        Sheet sheet = Sheet.createDefault();
//        Sheet.Set set = Sheet.createPropertiesSet();
//
//        try {
//
//            Property nameProp = new PropertySupport.Reflection(this.plot, String.class, "getName", null);
//            Property wktProp = new PropertySupport.Reflection(this.plot, String.class, "getWKT", null);
//
//            nameProp.setName("name");
//            wktProp.setName("wkt");
//
//            set.put(nameProp);
//            set.put(wktProp);
//
//        } catch (NoSuchMethodException ex) {
//            Exceptions.printStackTrace(ex);
//        }
//
//        sheet.put(set);
//        return sheet;
//    }
}
