/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.area.plot;

import java.awt.event.ActionEvent;
import java.beans.IntrospectionException;
import java.util.List;
import nl.cloudfarming.client.area.AreaService;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerAction;
import nl.cloudfarming.client.geoviewer.LayerEventKey;
import nl.cloudfarming.client.geoviewer.NewLayerEvent;
import nl.cloudfarming.client.model.Plot;
import nl.cloudfarming.client.model.PlotFile;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/**
 *
 * @author Timon Veenstra
 */
public class SaveAsPlotsAction extends LayerAction {

    public SaveAsPlotsAction() {
        putValue(NAME, NbBundle.getMessage(this.getClass(), "copy.import.to.plot.action"));
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Layer layer = Utilities.actionsGlobalContext().lookup(Layer.class);
        if (layer instanceof ImportLayer) {
            ImportLayer importLayer = (ImportLayer) layer;
            List<ShapeLayerObject> shapeLayerObjects = importLayer.getObjects();

            AreaService areaService = new PlotAreaService();
//            boolean plotLayerPresent = (areaService.findAllPlots().size() <= 0);


            PlotLayer plotLayer = new PlotLayer("plot.node.name");
            PlotFile plotFile = importLayer.getFile();


            // check if file has already been imported
            if (areaService.findPlotFileByNameChecksum(plotFile.getFileName(), plotFile.getChecksum()) == null) {
                areaService.persist(plotFile);

                for (ShapeLayerObject shapeLayerObject : shapeLayerObjects) {
                    try {
                        Shape shape = shapeLayerObject.getObject();
                        Plot plot = new Plot();
                        plot.setName(shape.getName());
                        plot.setWKT(shape.getGeometry().toText());
                        areaService.persist(plot);
                        plotLayer.addObject(new PlotLayerObject(plot));
                    } catch (IntrospectionException ex) {
                        Exceptions.printStackTrace(ex);
                    }
                }
                NewLayerEvent event = new NewLayerEvent(plotLayer);
                new LayerEventProducer().triggerEvent(event);
            }
        }
    }

    private class LayerEventProducer extends BaseGuiEventProducer {
        public LayerEventProducer() {
            super(LayerEventKey.class);
        }
    }
}
